/*
  Tool            XmasGift
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE v4.2 upwards
  Version         v1   14 Dec 2024


  This tool makes TSE draw one specific picture.


  INSTALLATION

  Copy XmasGift.s and XmasGift.dat to TSE's "mac" directory.

  Compile XmasGift.s there, for example by opening the file there in TSE,
  and by then applying the Macro -> Compile menu.


  USAGE

  Just execute the macro after compiling it,
  or use the menu Macro -> Execute, and enter:
    XmasGift

*/



// Start of compatibility restrictions and mitigations

/*
  If compiled for Linux or with a TSE version below TSE v4.2, then the compiler
  reports a syntax error and hilights the first applicable line below.
*/

#ifdef LINUX
  LINUX is not supported.
#endif

#ifndef WIN32
   16-bit versions of TSE are not supported. You need at least TSE v4.2.
#endif

#ifndef EDITOR_VERSION
   Editor Version is older than TSE v3.0. You need at least TSE v4.2.
#endif

#if EDITOR_VERSION < 4200h
   Editor Version is older than TSE v4.2. You need at least TSE v4.2.
#endif

// End of compatibility restrictions and mitigations.



proc give_gift()
  integer brush_attr             = 0
  integer brush_bg_index         = 0
  integer brush_bg_rgb_old       = 0
  integer bytes_read             = 0
  integer col                    = 0
  string  dat_fqn [MAXSTRINGLEN] = SplitPath(CurrMacroFilename(), _DRIVE_|_PATH_|_NAME_) + '.dat'
  string  data    [MAXSTRINGLEN] = ''
  integer handle                 = -1
  integer i                      = 0
  integer rgb                    = 0
  integer row                    = 0
  handle = fOpen(dat_fqn, _OPEN_READONLY_)
  if handle <> -1
    Set(Attr, Color(White ON Black))
    PopWinOpen(1, 1, Query(ScreenCols), Query(ScreenRows), 0, '', Color(White ON Black))
    ClrScr()
    brush_attr       = Color(Intense Black ON White)
    brush_bg_index   = brush_attr / 16
    brush_bg_rgb_old = GetColorTableValue(_BACKGROUND_, brush_bg_index)
    col              = 1
    row              = 1
    bytes_read       = fRead(handle, data, MAXSTRINGLEN)
    while bytes_read > 0
      for i = 1 to bytes_read by 3
        rgb = Asc(data[i]) * 65536 + Asc(data[i+1]) * 256 + Asc(data[i+2])
        SetColorTableValue(_BACKGROUND_, brush_bg_index, rgb)
        PutStrAttrXY(col, row, ' ', '', brush_attr)
        col = col + 1
        if col > 204
          row = row + 1
          col = 1
        endif
      endfor
      bytes_read = fRead(handle, data, MAXSTRINGLEN)
    endwhile
    GetKey()
    PopWinClose()
    SetColorTableValue(_BACKGROUND_, brush_bg_index, brush_bg_rgb_old)
  endif
end give_gift

proc x_mas_gift()
  integer old_CurrVideoMode              = Query(CurrVideoMode)
  integer old_Cursor                     = Set(Cursor        , OFF)
  integer old_Flags                      = 0
  string  old_FontName    [MAXSTRINGLEN] = ""
  integer old_PointSize                  = 0
  integer old_ShowHelpLine               = Set(ShowHelpLine  , OFF)
  integer old_ShowMainMenu               = Set(ShowMainMenu  , OFF)
  integer old_ShowStatusLine             = Set(ShowStatusLine, OFF)
  integer old_WhtSpc                     = FALSE
  string  old_WindowTitle [MAXSTRINGLEN] = ''
  integer org_id                         = GetBufferId()
  integer tmp_id                         = 0
  if isMacroLoaded("WhtSpc")
    old_WhtSpc = TRUE
    PurgeMacro("WhtSpc")
  endif
  GetFont(old_FontName, old_PointSize, old_Flags)
  SetFont("Courier New", 12, 0)
  while ResizeFont(-1)
  endwhile
  Set(CurrVideoMode, _MAX_LINES_COLS_)
  UpdateDisplay()
  old_WindowTitle = GetWindowTitle()
  SetWindowTitle('XmasGift')
  tmp_id = CreateTempBuffer()
  give_gift()
  Set(CurrVideoMode   , old_CurrVideoMode)
  Set(ShowMainMenu    , old_ShowMainMenu)
  Set(ShowHelpLine    , old_ShowHelpLine)
  Set(ShowStatusLine  , old_ShowStatusLine)
  Set(Cursor          , old_Cursor)
  SetFont(old_FontName, old_pointSize, old_Flags)
  Set(CurrVideoMode   , old_CurrVideoMode)
  SetWindowTitle(       old_WindowTitle)
  GotoBufferId(org_id)
  AbandonFile(tmp_id)
  UpdateDisplay()
  if old_WhtSpc
    LoadMacro("WhtSpc")
  endif
end x_mas_gift

proc Main()
  if isGUI()
    repeat
      x_mas_gift()
      Set(X1, Query(ScreenCols) / 2 - 10)
      Set(Y1, Query(ScreenRows) / 2 -  5)
      PushKey(<GreyCursorDown>)
    until YesNo("Stop XmasGift?") == 1
  else
    Warn('This tool only works with the GUI version of TSE.')
  endif
  PurgeMacro(CurrMacroFilename())
end Main

